@@ -13,36 +13,36 @@ BEGIN {
   my $seq = 0;
   if ($] > 5.009) {
     eval q(
-    package B::OP;
-    sub seq {
-      my $self = shift;
-      @_ ? $optimizer::seq = shift : $optimizer::seq;
-    }
-   );
+      package B::OP;
+      sub seq {
+        shift;
+        @_ ? $optimizer::seq = shift : $optimizer::seq;
+      }
+    );
   }
 }
 
 require DynaLoader;
-our $VERSION = '0.06';
-our @ISA=q(DynaLoader);
+our $VERSION = '0.07';
+our @ISA = q(DynaLoader);
 our %callbacks;
 bootstrap optimizer $VERSION;
 
 my ($file, $line) = ("unknown", "unknown");
 
 {
-sub preparewarn {
+sub _preparewarn {
     my $args = join '', @_;
     $args = "Something's wrong " unless $args;
     $args .= " at $file line $line.\n" unless substr($args, length($args) -1) eq "\n";
 }
 
-sub update {
+sub _update {
     my $cop = shift; $file = $cop->file; $line = $cop->line;
 }
 
-sub die (@) { CORE::die(preparewarn(@_)) }
-sub warn (@) { CORE::warn(preparewarn(@_)) }
+sub _die (@) { CORE::die(preparewarn(@_)) }
+sub _warn (@) { CORE::warn(preparewarn(@_)) }
 }
 
 sub import {
@@ -58,7 +58,7 @@ sub import {
     } elsif ($type eq "callback" or $type eq "extend" or $type eq "mine") {
         my $subref = shift;
         croak "Supplied callback was not a subref" unless ref $subref eq "CODE";
-        optimizer::install( sub { callbackoptimizer($_[0],$subref) }) if $type eq "callback";
+        optimizer::install( sub { callbackoptimizer($_[0], $subref) }) if $type eq "callback";
         optimizer::install( sub { optimizer::peepextend($_[0], $subref) }) if $type eq "extend";
         optimizer::install( $subref ) if $type eq "mine";
     } elsif ($type eq 'extend-c') {
@@ -78,8 +78,9 @@ sub callbackoptimizer {
     my ($op, $callback) = @_;
     while ($$op) {
 	$op->seq(optimizer::op_seqmax_inc());
-        update($op) if $op->isa("B::COP");
-        relocatetopad($op, $op->find_cv()) if $op->name eq "const"; # For thread safety
+        _update($op) if $op->isa("B::COP");
+	# crashes: wrong op_sv, strange cv
+        #_relocatetopad($op, $op->find_cv()) if $op->name eq "const"; # For thread safety
 
         $callback->($op);
         $op = $op->next;
@@ -98,15 +99,14 @@ sub peepextend {
     while ($$o) {
         #warn ("Trying op $o ($$o) -> ".$o->name."\n");
         if ($o->isa("B::COP")) {
-
             $o->seq(optimizer::op_seqmax_inc());
-            update($o); # For warnings
+            _update($o); # For warnings
 
         } elsif ($o->name eq "const") {
-            optimizer::die("Bareword ",$o->sv->sv, " not allowed while \"strict subs\" in use")
+            optimizer::_die("Bareword ",$o->sv->sv, " not allowed while \"strict subs\" in use")
                 if ($o->private & 8);
-
-            relocatetopad($o,$o->find_cv());
+	    # crashes: wrong op_sv, strange cv
+            #_relocatetopad($o, $o->find_cv());
             $o->seq(optimizer::op_seqmax_inc());
         } elsif ($o->name eq "concat") {
             if ($o->next && $o->next->name eq "stringify" and !($o->flags &64)) {
@@ -134,7 +134,7 @@ sub peepextend {
             $o->seq(optimizer::op_seqmax_inc());
             $o->other($o->other->next) while $o->other->name eq "null";
             peepextend($o->other, $callback); # Weee.
-        } elsif ($o->name =~ /^enter(loop|iter)/) {
+        } elsif ($o->name =~ /^enter(loop|iter|given|when)/) {
             $o->seq(optimizer::op_seqmax_inc());
             $o->redoop($o->redoop->next) while $o->redoop->name eq "null";
 	    peepextend($o->redoop, $callback);
@@ -151,8 +151,8 @@ sub peepextend {
             $o->seq(optimizer::op_seqmax_inc());
             if (${$o->next} and $o->next->name eq "nextstate" and
                 ${$o->next->sibling} and $o->next->sibling->type !~ /exit|warn|die/) {
-                optimizer::warn("Statement unlikely to be reached");
-                optimizer::warn("\t(Maybe you meant system() when you said exec()?)\n");
+                optimizer::_warn("Statement unlikely to be reached");
+                optimizer::_warn("\t(Maybe you meant system() when you said exec()?)\n");
             }
         } else {
             # Screw pseudohashes.
@@ -169,7 +169,6 @@ sub peepextend {
 
 1;
 __END__
-# Below is stub documentation for your module. You better edit it!
 
 =head1 NAME
 
@@ -191,7 +190,7 @@ optimizer - Write your own Perl optimizer, in Perl
   # Use a simple optimizer with callbacks for each op
   use optimizer callback => sub { .. }
 
-  # Completely implement your own optimizre
+  # Completely implement your own optimizer
   use optimizer mine => sub { ... }
 
   # use the standard optimizer with an extra callback
@@ -209,17 +208,16 @@ optimizer - Write your own Perl optimizer, in Perl
 This module allows you to replace the default Perl optree
 optimizer, C<peep>, with a Perl function of your own devising.
 
-It requires a Perl patched with the patch supplied with the
-module distribution; this patch allows the optimizer to be
+It requires a Perl > 5.8 or patched with the plugpeep patch supplied
+with the module distribution; this patch allows the optimizer to be
 pluggable and replaceable with a C function pointer. This module
-provides the glue between the C function and a Perl subroutine. It
-is hoped that the patch will be integrated into the Perl core at
-some point soon. This patch is integrated as of perl 5.8.
+provides the glue between the C function and a Perl subroutine.
+This patch was integrated as of perl 5.8.
 
 Your optimizer subroutine will be handed a C<B::OP>-derived object
 representing the first (NOT the root) op in the program. You are
 expected to be fluent with the C<B> module to know what to do with this.
-You can use C<B::Generate> to fiddle around with the optree you are
+You can use L<B::Generate> to fiddle around with the optree you are
 given, while traversing it in execution order.
 
 If you choose complete control over your optimizer, you B<must> assign
@@ -244,10 +242,110 @@ code block is compiled so you can do any arbitrary work on it use the
 C<sub-detect> option, you will be passed LEAVE* ops after the standard
 peep optimizer has been run, this minimises the risk for bugs as we
 use the standard one. The op tree you are handed is also stable so you
-are free to work on it. This is usefull if you are limited by
+are free to work on it. This is useful if you are limited by
 C<CHECK> and C<INIT> blocks as this works with string eval and
 C<require> aswell. Only one callback per package is allowed.
 
+PERL_DL_NONLAZY and B::Generate
+
+Note that optimizer do works fine on most platforms and perl versions,
+only on Windows some libperl functions are not exported anymore, which
+B::Generate needs.
+
+=head1 OPTIONS
+
+=over 4
+
+=item C
+
+Use Perl's default optimizer.
+
+  use optimizer 'C';
+
+=item perl
+
+Use a Perl implementation of Perl's default optimizer.
+
+  use optimizer 'perl';
+
+=item extend
+
+Use an extension of the default optimizer.
+The callback is called for every visited op.
+
+  use optimizer extend => sub {
+        warn "goto considered harmful" if $_[0]->name eq "goto"
+  }
+
+=item callback
+
+Use a simple optimizer with callbacks for each op.
+
+  use optimizer callback => sub { .. }
+
+=item mine
+
+Completely implement your own optimizer. You have to provide your own walker and peep.
+
+  use optimizer mine => sub { ... }
+
+=item extend-c
+
+Use the standard optimizer with an extra callback.
+This is the most compatible optimizer version.
+
+  use optimizer 'extend-c' => sub { print $_[0]->name() };
+
+=item sub-detect
+
+Don't provide a peep optimizer, rather get a callback
+after we are finished with every code block (I<sub>).
+You will be passed LEAVE* ops after the standard
+peep optimizer has been run, this minimises the risk
+for bugs as we use the standard one.
+
+  use optimizer 'sub-detect' => sub { print $_[0]->name() };
+
+=back
+
+=head1 HELPER FUNCTIONS
+
+=over 4
+
+=item callbackoptimizer (this, callback)
+
+The helper function for the option B<callback>.
+
+=item peepextend (this, callback)
+
+The helper function for the option B<extend>.
+
+->import('perl') uses B<peepextend> with an empty callback.
+
+=item c_extend_install
+
+The helper function for the option B<extend-c>.
+It uses the longish XS function C<c_extend_peep> as
+experimental peeper, and calls the user-side perl callback
+for each OP.
+
+=item c_sub_detect_install
+
+The XS helper function for the option B<sub-detect>.
+It installs C<c_sub_detect> as C<PL_peep>.
+B<c_sub_detect> calls all perl-side callbacks at any LEAVE op.
+
+=item unimport
+
+Override with an empty B<callbackoptimizer>, effectively disabling any
+installed optimizer.
+
+=back
+
+=head1 STATUS
+
+relocatetopad fails with threaded perls.
+
 =head1 5.10 Changes
 
 Since Perl 5.10 there are no op_seqmax and op_seq numbers in CORE
@@ -258,11 +356,11 @@ numbering, for $B::OP::seq also. This is not thread-safe.
 
   Simon Cozens, C<simon@cpan.org>
 
-Extended functionality and current maintainer:
+Extended functionality:
 
-  Arthur Bergman, C<abergman@cpan.org>
+  Artur Bergman, C<abergman@cpan.org>
 
-5.10 support by Reini Urban:
+5.10 support and current maintainer Reini Urban:
 
   Reini Urban, C<rurban@cpan.org>
 
